/**
 * Created on March 8, 2007
 * 
 * Tool for exporting CEM Business Process, Business Transaction statistic, Defects data and Incidents data through CEM webservices.
 * 
 * @copyright 1999-2007 CA.  All rights reserved.
 */

package com.export;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import com.ca.wily.cemwebservices.CEMWebServicesException;
import com.ca.wily.cemwebservices.DBusinessProcessStats;
import com.ca.wily.cemwebservices.DBusinessProcessStatsRS;
import com.ca.wily.cemwebservices.DBusinessTransactionStats;
import com.ca.wily.cemwebservices.DBusinessTransactionStatsRS;
import com.ca.wily.cemwebservices.DDefect;
import com.ca.wily.cemwebservices.DDefectRS;
import com.ca.wily.cemwebservices.DIncident;
import com.ca.wily.cemwebservices.DIncidentRS;
import com.ca.wily.cemwebservices.DTimeDuration;
import com.ca.wily.cemwebservices.EventsDataOutServiceSoapBindingStub;
import com.ca.wily.cemwebservices.StatisticsDataOutServiceSoapBindingStub;

/**
 * Wrapper class is used to invoke CEM webservice API. Since the number of
 * Statistic data that are returned in one call are limited to 500. To have this
 * wrapper class, user can retrieve all statistic data in one wrapper API call.
 * 
 * @author xinyi01
 */
public class CEMWSClientWrap {
    private StatisticsDataOutServiceSoapBindingStub sdo;
    private EventsDataOutServiceSoapBindingStub edo;
    private float[] throughputPercents;
    private float[] sizePercents;
    private float[] timePercents;

    /**
     * @param sdo
     *            StatisticDataOutServiceSoapBindingStub object
     * @param percents
     *            Float type of array, contains percentile value of transaction
     *            throughput, transaction size and transaction time.
     * @since 1.0
     */
    public CEMWSClientWrap(StatisticsDataOutServiceSoapBindingStub sdo,
            float[] throughputPercents, float[] sizePercents,
            float[] timePercents) {
        this.sdo = sdo;
        this.throughputPercents = throughputPercents;
        this.sizePercents = sizePercents;
        this.timePercents = timePercents;
    }

    public CEMWSClientWrap(EventsDataOutServiceSoapBindingStub edo) {
        this.edo = edo;
    }

    public DBusinessProcessStats getAggregatedBusinessProcessStats(long bpid,
            DTimeDuration interval) throws CEMWebServicesException,
            RemoteException {
        DBusinessProcessStats aggregatedBPStats = null;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            aggregatedBPStats = sdo.getAggregatedBusinessProcessStats(bpid,
                    interval);
        else
            aggregatedBPStats = sdo
                    .getAggregatedBusinessProcessStatsForPercentiles(bpid,
                            interval, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return aggregatedBPStats;
    }

    public DBusinessTransactionStats getAggregatedBusinessTransactionStats(
            long btid, DTimeDuration interval) throws CEMWebServicesException,
            RemoteException {
        DBusinessTransactionStats aggrBusinessTranStats = null;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            aggrBusinessTranStats = sdo.getAggregatedBusinessTransactionStats(
                    btid, interval);
        else
            aggrBusinessTranStats = sdo
                    .getAggregatedBusinessTransactionStatsForPercentiles(btid,
                            interval, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return aggrBusinessTranStats;
    }
    
    public DBusinessProcessStats getAggregatedBusinessProcessStatsForUser(
            long bpid, DTimeDuration interval, long userid)
            throws CEMWebServicesException, RemoteException 
    {
        DBusinessProcessStats statsDTO = null;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            statsDTO = sdo.getAggregatedBusinessProcessStatsForUser(bpid,
                    interval, userid);
        else
            statsDTO = sdo
                    .getAggregatedBusinessProcessStatsForUserForPercentiles(bpid,
                            interval, userid, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return statsDTO;
    }    

    public DBusinessProcessStats getAggregatedBusinessProcessStatsForUserGroup(
            long bpid, DTimeDuration interval, long ugid)
            throws CEMWebServicesException, RemoteException 
    {
        DBusinessProcessStats statsDTO = null;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            statsDTO = sdo.getAggregatedBusinessProcessStatsForUserGroup(bpid,
                    interval, ugid);
        else
            statsDTO = sdo
                    .getAggregatedBusinessProcessStatsForUserGroupForPercentiles(bpid,
                            interval, ugid, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return statsDTO;
    }
    
    public DBusinessTransactionStats getAggregatedBusinessTransactionStatsForUserGroup(
            long btid, DTimeDuration interval, long ugid)
            throws CEMWebServicesException, RemoteException 
    {
        DBusinessTransactionStats statsDTO;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            statsDTO = sdo.getAggregatedBusinessTransactionStatsForUserGroup(btid,
                    interval, ugid);
        else
            statsDTO = sdo
                    .getAggregatedBusinessTransactionStatsForUserGroupForPercentiles(
                            btid, interval, ugid, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return statsDTO;
    }
    
    public DBusinessTransactionStats getAggregatedBusinessTransactionStatsForUser(
            long btid, DTimeDuration interval, long userid)
            throws CEMWebServicesException, RemoteException 
    {
        DBusinessTransactionStats statsDTO;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            statsDTO = sdo.getAggregatedBusinessTransactionStatsForUser(btid,
                    interval, userid);
        else
            statsDTO = sdo
                    .getAggregatedBusinessTransactionStatsForUserForPercentiles(
                            btid, interval, userid, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        return statsDTO;
    }

    /**
     * Gets statistic for all Business Processes in a specified time range,
     * aggregated accross all User Groups.
     * 
     * @param bpid
     *            Id of the business process definition
     * @param interval
     *            Statistics data aggregation type
     * @return A list of DBusinessProcessStats objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DBusinessProcessStats> getAllBusinessProcessStats(long bpid,
            DTimeDuration interval) throws CEMWebServicesException,
            RemoteException {
        List<DBusinessProcessStats> results = new ArrayList<DBusinessProcessStats>();

        /*
         * data amount limitation from CEM default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DBusinessProcessStatsRS allBPStatsRS;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            allBPStatsRS = sdo.getAllBusinessProcessStats(bpid, interval, 0);
        else
            allBPStatsRS = sdo.getAllBusinessProcessStatsForPercentiles(bpid,
                    interval, 0, this.throughputPercents, this.sizePercents,
                    this.timePercents);

        int start = allBPStatsRS.getNextStartIndex();
        DBusinessProcessStats[] allBPStats = allBPStatsRS
                .getDBusinessProcessStats();
        for (int m = 0; m < allBPStats.length; m++) {
            results.add(allBPStats[m]);
        }

        while (start != -1) {
            DBusinessProcessStatsRS tempRS;

            if (!isPercentile)
                tempRS = sdo.getAllBusinessProcessStats(bpid, interval, start);
            else
                tempRS = sdo.getAllBusinessProcessStatsForPercentiles(bpid,
                        interval, start, this.throughputPercents,
                        this.sizePercents, this.timePercents);

            DBusinessProcessStats[] temp = tempRS.getDBusinessProcessStats();
            for (int m = 0; m < temp.length; m++) {
                results.add(temp[m]);
            }
            start = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets statistic for all Business Transactions in a specified time range,
     * aggregated accross all User Groups.
     * 
     * @param btid
     *            Id of business transaction definition
     * @param interval
     *            Statistics data aggregation type
     * @return A list of DBusinessTransactionStats objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DBusinessTransactionStats> getAllBusinessTransactionStats(
            long btid, DTimeDuration interval) throws CEMWebServicesException,
            RemoteException {
        List<DBusinessTransactionStats> results = new ArrayList<DBusinessTransactionStats>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DBusinessTransactionStatsRS allBusinessTranStatsRS;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            allBusinessTranStatsRS = sdo.getAllBusinessTransactionStats(btid,
                    interval, 0);
        else
            allBusinessTranStatsRS = sdo
                    .getAllBusinessTransactionStatsForPercentiles(btid,
                            interval, 0, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        int start = allBusinessTranStatsRS.getNextStartIndex();
        DBusinessTransactionStats[] allBusinessTranStats = allBusinessTranStatsRS
                .getDBusinessTransactionStats();
        for (int m = 0; m < allBusinessTranStats.length; m++) {
            results.add(allBusinessTranStats[m]);
        }

        while (start != -1) {
            DBusinessTransactionStatsRS tempRS;

            if (!isPercentile)
                tempRS = sdo.getAllBusinessTransactionStats(btid, interval,
                        start);
            else
                tempRS = sdo.getAllBusinessTransactionStatsForPercentiles(btid,
                        interval, start, this.throughputPercents,
                        this.sizePercents, this.timePercents);

            DBusinessTransactionStats[] temp = tempRS
                    .getDBusinessTransactionStats();
            for (int m = 0; m < temp.length; m++) {
                results.add(temp[m]);
            }
            start = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets statistic for all Business Processes in a specified time range,
     * aggregated for a specified User Group.
     * 
     * @param bpid
     *            Id of business process definition
     * @param interval
     *            Statistics data aggregation type
     * @param ugid
     *            Id of user group
     * @return A list of DBusinessProcessStats objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DBusinessProcessStats> getBusinessProcessStatsForUserGroup(
            long bpid, DTimeDuration interval, long ugid)
            throws CEMWebServicesException, RemoteException {
        List<DBusinessProcessStats> results = new ArrayList<DBusinessProcessStats>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DBusinessProcessStatsRS busProStatsRS;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            busProStatsRS = sdo.getBusinessProcessStatsForUserGroup(bpid,
                    interval, ugid, 0);
        else
            busProStatsRS = sdo
                    .getBusinessProcessStatsForUserGroupForPercentiles(bpid,
                            interval, ugid, 0, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        int start = busProStatsRS.getNextStartIndex();
        DBusinessProcessStats[] busProStats = busProStatsRS
                .getDBusinessProcessStats();
        for (int m = 0; m < busProStats.length; m++) {
            results.add(busProStats[m]);
        }

        while (start != -1) {
            DBusinessProcessStatsRS tempRS;

            if (!isPercentile)
                tempRS = sdo.getBusinessProcessStatsForUserGroup(bpid,
                        interval, ugid, start);
            else
                tempRS = sdo.getBusinessProcessStatsForUserGroupForPercentiles(
                        bpid, interval, ugid, start, this.throughputPercents,
                        this.sizePercents, this.timePercents);

            DBusinessProcessStats[] temp = tempRS.getDBusinessProcessStats();
            for (int m = 0; m < temp.length; m++) {
                results.add(temp[m]);
            }
            start = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets statistic for all Business Transactions in a specified time range,
     * aggregated for a specified User Group.
     * 
     * @param btid
     *            Id of business transaction definition
     * @param interval
     *            Statistics data aggregation type
     * @param ugid
     *            Id of user group
     * @return A list of DBusinessTransactionStats objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DBusinessTransactionStats> getBusinessTransactionStatsForUserGroup(
            long btid, DTimeDuration interval, long ugid)
            throws CEMWebServicesException, RemoteException {
        List<DBusinessTransactionStats> results = new ArrayList<DBusinessTransactionStats>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DBusinessTransactionStatsRS busTranStatsRS;
        boolean isPercentile = false;

        // default to 50% if not specified
        if (this.throughputPercents != null || this.sizePercents != null
                || this.timePercents != null)
            isPercentile = true;

        if (isPercentile && this.throughputPercents == null) {
            this.throughputPercents = new float[1];
            this.throughputPercents[0] = 0.5f;
        }
        if (isPercentile && this.sizePercents == null) {
            this.sizePercents = new float[1];
            this.sizePercents[0] = 0.5f;
        }
        if (isPercentile && this.timePercents == null) {
            this.timePercents = new float[1];
            this.timePercents[0] = 0.5f;
        }

        if (!isPercentile)
            busTranStatsRS = sdo.getBusinessTransactionStatsForUserGroup(btid,
                    interval, ugid, 0);
        else
            busTranStatsRS = sdo
                    .getBusinessTransactionStatsForUserGroupForPercentiles(
                            btid, interval, ugid, 0, this.throughputPercents,
                            this.sizePercents, this.timePercents);

        int start = busTranStatsRS.getNextStartIndex();
        DBusinessTransactionStats[] busTranStats = busTranStatsRS
                .getDBusinessTransactionStats();
        for (int m = 0; m < busTranStats.length; m++) {
            results.add(busTranStats[m]);
        }

        while (start != -1) {
            DBusinessTransactionStatsRS tempRS;

            if (!isPercentile)
                tempRS = sdo.getBusinessTransactionStatsForUserGroup(btid,
                        interval, ugid, start);
            else
                tempRS = sdo
                        .getBusinessTransactionStatsForUserGroupForPercentiles(
                                btid, interval, ugid, start,
                                this.throughputPercents, this.sizePercents,
                                this.timePercents);

            DBusinessTransactionStats[] temp = tempRS
                    .getDBusinessTransactionStats();
            for (int m = 0; m < temp.length; m++) {
                results.add(temp[m]);
            }
            start = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets the Defects that happened in a specified time range, and are
     * associated with a specified Business Process.
     * 
     * @param dTimeDuration
     *            The time range and the aggregation interval
     * @param bpid
     *            The id of business transaction
     * @return A list of DDefect object
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DDefect> getDefectsByBusinessProcess(
            DTimeDuration dTimeDuration, long bpid)
            throws CEMWebServicesException, RemoteException {
        List<DDefect> results = new ArrayList<DDefect>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DDefectRS defectRS = edo.getDefectsByBusinessProcess(bpid,
                dTimeDuration.getStartTime(), dTimeDuration.getEndTime(), 0);
        DDefect[] defects = defectRS.getDDefect();
        int nextStart = defectRS.getNextStartIndex();

        for (int i = 0; i < defects.length; i++) {
            results.add(defects[i]);
        }

        while (nextStart != -1) {
            DDefectRS tempRS = edo.getDefectsByBusinessProcess(bpid,
                    dTimeDuration.getStartTime(), dTimeDuration.getEndTime(),
                    nextStart);
            DDefect[] temp = tempRS.getDDefect();

            for (int i = 0; i < temp.length; i++) {
                results.add(temp[i]);
            }

            nextStart = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets the Defects that happened in a specified time range, and are
     * associated with a specified Business Transaction.
     * 
     * @param dTimeDuration
     *            The time range and the aggregation interval
     * @param btid
     *            The id of business transaction
     * @return A list of DDefect objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DDefect> getDefectsByBusinessTransaction(
            DTimeDuration dTimeDuration, long btid)
            throws CEMWebServicesException, RemoteException {
        List<DDefect> results = new ArrayList<DDefect>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DDefectRS defectRS = edo.getDefectsByBusinessTransaction(btid,
                dTimeDuration.getStartTime(), dTimeDuration.getEndTime(), 0);
        DDefect[] defects = defectRS.getDDefect();
        int nextStart = defectRS.getNextStartIndex();

        for (int i = 0; i < defects.length; i++) {
            results.add(defects[i]);
        }

        while (nextStart != -1) {
            DDefectRS tempRS = edo.getDefectsByBusinessTransaction(btid,
                    dTimeDuration.getStartTime(), dTimeDuration.getEndTime(),
                    nextStart);
            DDefect[] temp = tempRS.getDDefect();

            for (int i = 0; i < temp.length; i++) {
                results.add(temp[i]);
            }
            nextStart = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets the Defects that happened in a specified time range, and are
     * associated with a specified User Group
     * 
     * @param dTimeDuration
     *            The time range and the aggregation interval
     * @param ugid
     *            The id of user group
     * @return A list of DDefect objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DDefect> getDefectsByUserGroup(DTimeDuration dTimeDuration,
            long ugid) throws CEMWebServicesException, RemoteException {
        List<DDefect> resultsRS = new ArrayList<DDefect>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DDefectRS defectRS = edo.getDefectsByUserGroup(ugid, dTimeDuration
                .getStartTime(), dTimeDuration.getEndTime(), 0);
        int nextStart = defectRS.getNextStartIndex();
        DDefect[] defects = defectRS.getDDefect();
        for (int i = 0; i < defects.length; i++) {
            resultsRS.add(defects[i]);
        }

        while (nextStart != -1) {
            DDefectRS tempRS = edo.getDefectsByUserGroup(ugid, dTimeDuration
                    .getStartTime(), dTimeDuration.getEndTime(), nextStart);
            DDefect[] temp = tempRS.getDDefect();

            for (int i = 0; i < temp.length; i++) {
                resultsRS.add(temp[i]);
            }
            nextStart = tempRS.getNextStartIndex();
        }

        return resultsRS;
    }

    /**
     * <p>
     * Gets the Defects that happened in a specified time range, and specified
     * type.
     * <p>
     * 
     * <p>
     * Possible values for Defect type are:
     * <p>
     * 1: (DEFECT_TYPE_SLOW_TRAN_TIME) 2: (DEFECT_TYPE_FAST_TRAN_TIME) 3:
     * (DEFECT_TYPE_HIGH_THROUGHPUT) 4: (DEFECT_TYPE_LOW_THROUGHPUT) 5:
     * (DEFECT_TYPE_LARGE_TRAN_SIZE) 6: (DEFECT_TYPE_SMALL_TRAN_SIZE) 8:
     * (DEFECT_TYPE_HTTP_STATUS_ERROR) 9: (DEFECT_TYPE_MISSING_SUB_PART) 10:
     * (DEFECT_TYPE_CONTENT_ERROR) 11: (DEFECT_TYPE_MISSING_RESPONSE) 16:
     * (DEFECT_TYPE_PARTIAL_RESPONSE)
     * 
     * <p>
     * You can also query for Defects which are not of a certain type by
     * prefixing by "!". For example, to fetch all defects of type Slow
     * Transaction Time, the type would be "1", and to fetch all defects which
     * are not of type Slow Transaction Time, the type would be "!1".
     * <p>
     * 
     * @param dTimeDuration
     *            The time range and the aggregation interval
     * @param type
     *            Defect type
     * @return A list of DDefect objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DDefect> getDefectsByType(DTimeDuration dTimeDuration,
            String type) throws CEMWebServicesException, RemoteException {
        List<DDefect> results = new ArrayList<DDefect>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DDefectRS defectsRS = edo.getDefectsByType(type, dTimeDuration
                .getStartTime(), dTimeDuration.getEndTime(), 0);
        int nextStart = defectsRS.getNextStartIndex();
        DDefect[] defects = defectsRS.getDDefect();

        for (int i = 0; i < defects.length; i++) {
            results.add(defects[i]);
        }

        while (nextStart != -1) {
            DDefectRS tempRS = edo.getDefectsByType(type, dTimeDuration
                    .getStartTime(), dTimeDuration.getEndTime(), nextStart);
            DDefect[] temp = tempRS.getDDefect();

            for (int i = 0; i < temp.length; i++) {
                results.add(temp[i]);
            }
            nextStart = tempRS.getNextStartIndex();
        }

        return results;
    }

    /**
     * Gets the Incidents that happened in a specified time range.
     * 
     * @param dTimeDuration
     *            The time range and the aggregation interval
     * @return A list of DIncident objects
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @since 1.0
     */
    public List<DIncident> getIncidentsByTime(DTimeDuration dTimeDuration)
            throws CEMWebServicesException, RemoteException {
        List<DIncident> results = new ArrayList<DIncident>();

        /*
         * data amount limitation from CEM the default amount is 500 records.
         * 
         * for the initial call, always pass "0"
         */
        DIncidentRS incidentRS = edo.getIncidentsByTime(dTimeDuration
                .getStartTime(), dTimeDuration.getEndTime(), 0);
        int nextStart = incidentRS.getNextStartIndex();
        DIncident[] incidents = incidentRS.getDIncident();

        for (int i = 0; i < incidents.length; i++) {
            results.add(incidents[i]);
        }

        while (nextStart != -1) {
            DIncidentRS tempRS = edo.getIncidentsByTime(dTimeDuration
                    .getStartTime(), dTimeDuration.getEndTime(), nextStart);
            DIncident[] temp = tempRS.getDIncident();

            for (int i = 0; i < temp.length; i++) {
                results.add(temp[i]);
            }
            nextStart = tempRS.getNextStartIndex();
        }

        return results;
    }

}
