/**
 * Created on April 10, 2007
 * 
 * report for getting all Business Transactions Statistic data in a give day
 * 
 * @copyright 1999-2007 CA.  All rights reserved.
 */

package com.export;

import java.rmi.RemoteException;
import java.util.List;

import com.ca.wily.cemwebservices.CEMWebServicesException;
import com.ca.wily.cemwebservices.ConfigurationDataOutServiceSoapBindingStub;
import com.ca.wily.cemwebservices.DApplicationDefinition;
import com.ca.wily.cemwebservices.DBusinessProcessDefinition;
import com.ca.wily.cemwebservices.DBusinessProcessStats;
import com.ca.wily.cemwebservices.DBusinessTransactionDefinition;
import com.ca.wily.cemwebservices.DBusinessTransactionStats;
import com.ca.wily.cemwebservices.DDefect;
import com.ca.wily.cemwebservices.DIncident;
import com.ca.wily.cemwebservices.DTimeDuration;
import com.ca.wily.cemwebservices.DUser;
import com.ca.wily.cemwebservices.DUserGroup;
import com.ca.wily.cemwebservices.EventsDataOutServiceSoapBindingStub;
import com.ca.wily.cemwebservices.StatisticsDataOutServiceSoapBindingStub;

/**
 * Contains all samples for using CEM webservice API 
 * to retrieve statistic data, defects data and incidents data.
 * 
 * @author xinyi01
 */
public class Samples 
{
    
    public static void getAggregatedBPStatisticData(ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
    
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            String[][] keyPairs = new String[2][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(allBusinessProcessDef[i].getId());
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
                
            DBusinessProcessStats result = 
                wsClientwrap.getAggregatedBusinessProcessStats(allBusinessProcessDef[i].getId(), dTimeDuration);
            
            if( result != null) {
                Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
            }
        }
    }  
    
    public static void getAggregatedBTStatisticData( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {   
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            
            DBusinessTransactionDefinition[] busTranDef = 
                cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++){
                long btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = btName;
                
                DBusinessTransactionStats result = wsClientwrap.getAggregatedBusinessTransactionStats(btid, dTimeDuration);
                
                if (result != null) {
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);
                }
            }   
        }
    }
    
    public static void getAggregatedBPStatisticDataByBusinessProcess(
            ConfigurationDataOutServiceSoapBindingStub cdo,  
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess) throws CEMWebServicesException,RemoteException,Exception
    {
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long bpid = bpDef.getId();
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        String[][] keyPairs = new String[2][2];
        keyPairs[0][0] = String.valueOf(appId);
        keyPairs[0][1]= appName;
        keyPairs[1][0] = String.valueOf(bpid);
        keyPairs[1][1]= businessProcess;
        
        DBusinessProcessStats result = wsClientwrap.getAggregatedBusinessProcessStats(bpid, dTimeDuration);
        
        if (result != null)
            Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
    }
    
    public static void getAggregatedBTStatisticDataByBusinessTransaction( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String transaction) throws CEMWebServicesException,RemoteException,Exception
    {
        long btid = 0l;
        boolean ismore = false;
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            boolean isfound = false;
            
            for (int j = 0; j < busTranDef.length; j++){
                btid = busTranDef[j].getId();
                if (busTranDef[j].getName().compareTo(transaction) == 0){
                    isfound = true;
                    ismore = true;
                    break;
                }
            }
            
            if (isfound) {
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = transaction;
                
                DBusinessTransactionStats result = wsClientwrap.getAggregatedBusinessTransactionStats(btid, dTimeDuration);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);
            }   
        }
        
        if (!ismore)
            throw new Exception("business transaction name: " + transaction + " is not found.");
    }
    
    public static void getAggregatedBPStatisticDataByUser(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String user) throws CEMWebServicesException,RemoteException,Exception
    {
        long userid = findUserId(cdo, user);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);        
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[3][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(userid);
            keyPairs[2][1] = user;
            
            DBusinessProcessStats result = wsClientwrap.getAggregatedBusinessProcessStatsForUser(bpid, dTimeDuration, userid);
            
            if (result != null)
                Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
                
        }
    }
    
    public static void getAggregatedBPStatisticDataByUserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);        
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[3][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessProcessStats result = wsClientwrap.getAggregatedBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
            
            if (result != null)
                Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
                
        }
    }
    
    public static void getAggregatedBTStatisticDataByUser(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String user) throws CEMWebServicesException,RemoteException,Exception
    {
        long userid = findUserId(cdo, user);
        
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientWrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){         
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(userid);
            keyPairs[2][1] = user;
            
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++) {
                long btid = busTranDef[j].getId();  
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = busTranDef[j].getName();
                
                DBusinessTransactionStats result = wsClientWrap.getAggregatedBusinessTransactionStatsForUser(btid, dTimeDuration, userid);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);        
            }
        }
    }
    
    public static void getAggregatedBTStatisticDataByUserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientWrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){         
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++) {
                long btid = busTranDef[j].getId();  
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = busTranDef[j].getName();
                
                DBusinessTransactionStats result = wsClientWrap.getAggregatedBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);        
            }
        }
    }
    
    public static void getAggregatedBPStatisticDataByBusinessProcess_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long bpid = bpDef.getId();
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        
        long ugid = findUserGroupId(cdo, userGroup);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        String[][] keyPairs = new String[3][2];
        keyPairs[0][0] = String.valueOf(appId);
        keyPairs[0][1] = appName;
        keyPairs[1][0] = String.valueOf(bpid);
        keyPairs[1][1] = businessProcess;
        keyPairs[2][0] = String.valueOf(ugid);
        keyPairs[2][1] = userGroup;
        
        DBusinessProcessStats result = wsClientwrap.getAggregatedBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
        
        if (result!= null)
            Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
    }
    
    public static void getAggregatedBTStatisticDataByBusinessTransaction_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessTransaction,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        boolean isfound = false;
        long btid = 0L;
        boolean ismore = false;
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            isfound = false;
            
            for (int j = 0; j < busTranDef.length; j++){
                btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                if (businessTransaction.compareTo(btName) == 0){
                    isfound = true;
                    ismore = true;
                    break;
                }
            }
            
            if (isfound) {
                String[][] keyPairs = new String[4][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(ugid);
                keyPairs[2][1] = userGroup;
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = businessTransaction;

                DBusinessTransactionStats result = 
                    wsClientwrap.getAggregatedBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);    
            }
        }
        
        if (!ismore)
            throw new Exception("business transaction name: " + businessTransaction + " is not found."); 
    }
    
    public static void getAggregatedBTStatisticDataByBusinessProcess_BusinessTransaction_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess,
            String businessTransaction,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        long bpid = bpDef.getId();
        
        boolean isfound = false;
        long btid = 0L;
        String btName;
        DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
        for (int i = 0; i < busTranDef.length; i++){
            btid = busTranDef[i].getId();
            btName = busTranDef[i].getName();
            
            if (businessTransaction.compareTo(btName) == 0){
                isfound = true;
                break;
            }
        }
        
        if (isfound) {
            CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(appId);
            keyPairs[0][1] = appName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = businessProcess;
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            keyPairs[3][0] = String.valueOf(btid);
            keyPairs[3][1] = businessTransaction;
            
            DBusinessTransactionStats result = 
                wsClientwrap.getAggregatedBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
            
            if (result != null)
                Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);    
            
        }else
            throw new Exception("business transaction name : " + businessTransaction + " is not found.");               
    }
    
    public static void getAggregatedBPStatisticDataByApplication(ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            String applicationName,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
    
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            String[][] keyPairs = new String[2][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(allBusinessProcessDef[i].getId());
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
                
           DBusinessProcessStats result = 
                wsClientwrap.getAggregatedBusinessProcessStats(allBusinessProcessDef[i].getId(), dTimeDuration);
                
            if (result != null)
                Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
        }
    }   
    
    public static void getAggregatedBTStatisticDataByApplication( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            String applicationName,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {   
        long appid = findApplicationId(cdo, applicationName);
        
        // get all BusinessProcessDefinitions belonging to the application
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            
            DBusinessTransactionDefinition[] busTranDef = 
                cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++){
                long btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appid);
                keyPairs[0][1] = applicationName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = btName;
                
                DBusinessTransactionStats result = 
                    wsClientwrap.getAggregatedBusinessTransactionStats(btid, dTimeDuration);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);
            }   
        }
    }
    
    public static void getAggregatedBPStatisticDataByApplication_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String applicationName,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);        
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[3][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessProcessStats result = 
                wsClientwrap.getAggregatedBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
            
            if (result != null)
                Util.writeBusinessProcessStatisticToDataFile(dTimeDuration, keyPairs, result);
                
        }
    }
    
    public static void getAggregatedBTStatisticDataByApplication_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String applicationName,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        
        CEMWSClientWrap wsClientWrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){         
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++) {
                long btid = busTranDef[j].getId();  
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = busTranDef[j].getName();
                
                DBusinessTransactionStats result = 
                    wsClientWrap.getAggregatedBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (result != null)
                    Util.writeBusinessTransactionStatisticToDataFile(dTimeDuration, keyPairs, result);        
            }
        }
    }
    
    /**
     * Gets all business process statistic data for the given application from CEM StatisticDataOutService web service.
     * @param cdo 
     *          ConfigurationDataOutServiceSoapBindingStub object
     * @param sdo 
     *          StatisticDataOutServiceSoapBindingStub object.
     * @param applicationName The name of the application
     * 
     * @param dTimeDuration
     *          The time range and the aggregation interval
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time         
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBPStatisticDataByApplication(ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            String applicationName,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
    
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            String[][] keyPairs = new String[2][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(allBusinessProcessDef[i].getId());
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
                
            List<DBusinessProcessStats> results = 
                wsClientwrap.getAllBusinessProcessStats(allBusinessProcessDef[i].getId(), dTimeDuration);
                
            if (results.size() > 0)
                Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
        }
    }   
    
    /**
     * Gets all business process statistic data from CEM StatisticDataOutService web service.
     * @param cdo 
     *             ConfigurationDataOutServiceSoapBindingStub object
     * @param sdo 
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getAllBPStatisticData(ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
    
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            String[][] keyPairs = new String[2][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(allBusinessProcessDef[i].getId());
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
                
            List<DBusinessProcessStats> results = 
                wsClientwrap.getAllBusinessProcessStats(allBusinessProcessDef[i].getId(), dTimeDuration);
            
            if (results.size() > 0)
                Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
        }
    }    
    
       /**
     * Gets all business transaction statistic data for the given application from CEM StatisticDataOutService web service
     * @param cdo
     *          ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *          StatisticDataOutServiceSoapBindingStub object.
     * @param applicationName The name of the application
     * @param dTimeDuration
     *          The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByApplication( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            String applicationName,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {   
        long appid = findApplicationId(cdo, applicationName);
        
        // get all BusinessProcessDefinitions belonging to the application
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            
            DBusinessTransactionDefinition[] busTranDef = 
                cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++){
                long btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appid);
                keyPairs[0][1] = applicationName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = btName;
                
                List<DBusinessTransactionStats> results = 
                    wsClientwrap.getAllBusinessTransactionStats(btid, dTimeDuration);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);
            }   
        }
    }
    
    /**
     * Gets all business transaction statistic data from CEM StatisticDataOutService web service
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getAllBTStatisticData( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents) throws CEMWebServicesException,RemoteException,Exception
    {    
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++) {
            
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            
            DBusinessTransactionDefinition[] busTranDef = 
                cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++){
                long btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = btName;
                
                List<DBusinessTransactionStats> results = 
                    wsClientwrap.getAllBusinessTransactionStats(btid, dTimeDuration);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);
            }    
        }
    }
    
    
    /**
     * Gets business process statistic data for a specified business process 
     *             from CEM StatisticDataOutService web service
     * @param cdo 
     *             ConfigurationDataOutServiceSoapBindingStub object
     * @param sdo 
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param businessProcess
     *             The name of business process
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBPStatisticDataByBusinessProcess(
            ConfigurationDataOutServiceSoapBindingStub cdo,  
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess) throws CEMWebServicesException,RemoteException,Exception
    {
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long bpid = bpDef.getId();
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        String[][] keyPairs = new String[2][2];
        keyPairs[0][0] = String.valueOf(appId);
        keyPairs[0][1]= appName;
        keyPairs[1][0] = String.valueOf(bpid);
        keyPairs[1][1]= businessProcess;
        
        List<DBusinessProcessStats> results = 
            wsClientwrap.getAllBusinessProcessStats(bpid, dTimeDuration);
        
        if (results.size() > 0)
            Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
    }
    
    
    /**
     * Gets business transaction statistic data for specified transaction 
     *             from CEM StatisticDataOutService web service
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param transaction
     *             The name of transaction
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByBusinessTransaction( 
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String transaction) throws CEMWebServicesException,RemoteException,Exception
    {
        long btid = 0l;
        boolean ismore = false;
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            boolean isfound = false;
            
            for (int j = 0; j < busTranDef.length; j++){
                btid = busTranDef[j].getId();
                if (busTranDef[j].getName().compareTo(transaction) == 0){
                    isfound = true;
                    ismore = true;
                    break;
                }
            }
            
            if (isfound) {
                String[][] keyPairs = new String[3][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(btid);
                keyPairs[2][1] = transaction;
                
                List<DBusinessTransactionStats> results =
                    wsClientwrap.getAllBusinessTransactionStats(btid, dTimeDuration);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);
            }    
        }
        
        if (!ismore)
            throw new Exception("business transaction name: " + transaction + " is not found.");
    }


    /**
     * Gets business process statistic data for specified user group
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param userGroup
     *             The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBPStatisticDataByUserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);        
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[3][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            List<DBusinessProcessStats> results = 
                wsClientwrap.getBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
            
            if (results.size() > 0)
                Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
                
        }
    }

    /**
     * Gets business process statistic data for specified application and user group
     * @param cdo
     *          ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *          StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *          The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param applicationName
     *          The name of application
     * @param userGroup
     *          The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBPStatisticDataByApplication_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String applicationName,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);        
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[3][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            List<DBusinessProcessStats> results = 
                wsClientwrap.getBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
            
            if (results.size() > 0)
                Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
                
        }
    }

    /**
     * Gets business transaction statistic data for specified user group
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param userGroup
     *             The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByUserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        CEMWSClientWrap wsClientWrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){            
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(allBusinessProcessDef[i].getApplicationId());
            keyPairs[0][1] = allBusinessProcessDef[i].getApplicationName();
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++) {
                long btid = busTranDef[j].getId();    
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = busTranDef[j].getName();
                
                List<DBusinessTransactionStats> results = 
                    wsClientWrap.getBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);        
            }
        }
    }
    
    /**
     * Gets business transaction statistic data for specified application and user group
     * @param cdo
     *          ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *          StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *          The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time    
     * @param applicationName The name of the application    
     * @param userGroup
     *          The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByApplication_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String applicationName,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long appid = findApplicationId(cdo, applicationName);
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions, userGroups
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getBusinessProcessDefinitionsByAppId(appid);
        
        CEMWSClientWrap wsClientWrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){         
            long bpid = allBusinessProcessDef[i].getId();
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(appid);
            keyPairs[0][1] = applicationName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = allBusinessProcessDef[i].getName();
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            
            for (int j = 0; j < busTranDef.length; j++) {
                long btid = busTranDef[j].getId();  
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = busTranDef[j].getName();
                
                List<DBusinessTransactionStats> results = 
                    wsClientWrap.getBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);        
            }
        }
    }

    /**
     * Gets business process statistic data for specified business process, user group
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param businessProcess
     *             The name of business process
     * @param userGroup
     *             The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBPStatisticDataByBusinessProcess_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long bpid = bpDef.getId();
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        
        long ugid = findUserGroupId(cdo, userGroup);
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        String[][] keyPairs = new String[3][2];
        keyPairs[0][0] = String.valueOf(appId);
        keyPairs[0][1] = appName;
        keyPairs[1][0] = String.valueOf(bpid);
        keyPairs[1][1] = businessProcess;
        keyPairs[2][0] = String.valueOf(ugid);
        keyPairs[2][1] = userGroup;
        
        List<DBusinessProcessStats> results = 
            wsClientwrap.getBusinessProcessStatsForUserGroup(bpid, dTimeDuration, ugid);
        
        if (results.size() > 0)
            Util.writeBusinessProcessStatisticToDataFile(keyPairs, results);
    }

    
    /**
     * Gets business transaction statistic data for specified transaction and specified userGroup
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param businessTransaction
     *             The name of transaction
     * @param userGroup
     *             The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByBusinessTransaction_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessTransaction,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        
        boolean isfound = false;
        long btid = 0L;
        boolean ismore = false;
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
        for (int i = 0; i < allBusinessProcessDef.length; i++){
            long appId = allBusinessProcessDef[i].getApplicationId();
            String appName = allBusinessProcessDef[i].getApplicationName();
            long bpid = allBusinessProcessDef[i].getId();
            String bpName = allBusinessProcessDef[i].getName();
            DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
            isfound = false;
            
            for (int j = 0; j < busTranDef.length; j++){
                btid = busTranDef[j].getId();
                String btName = busTranDef[j].getName();
                
                if (businessTransaction.compareTo(btName) == 0){
                    isfound = true;
                    ismore = true;
                    break;
                }
            }
            
            if (isfound) {
                String[][] keyPairs = new String[4][2];
                keyPairs[0][0] = String.valueOf(appId);
                keyPairs[0][1] = appName;
                keyPairs[1][0] = String.valueOf(bpid);
                keyPairs[1][1] = bpName;
                keyPairs[2][0] = String.valueOf(ugid);
                keyPairs[2][1] = userGroup;
                keyPairs[3][0] = String.valueOf(btid);
                keyPairs[3][1] = businessTransaction;

                List<DBusinessTransactionStats> results = 
                    wsClientwrap.getBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
                
                if (results.size() > 0)
                    Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);    
            }
        }
        
        if (!ismore)
            throw new Exception("business transaction name: " + businessTransaction + " is not found."); 
    }

    
    /**
     * Gets business transaction statistic data for specified transaction, specified business process and specified userGroup
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param sdo
     *             StatisticDataOutServiceSoapBindingStub object.
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param throughputPercents
     *          The percentiles of transaction throughput
     * @param sizePercents
     *          The percentiles of transaction size
     * @param timePercents
     *          The percentiles of transaction time        
     * @param businessProcess
     *             The name of business process
     * @param businessTransaction
     *             The name of transaction
     * @param userGroup
     *             The name of userGroup
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getBTStatisticDataByBusinessProcess_BusinessTransaction_UserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            StatisticsDataOutServiceSoapBindingStub sdo,
            DTimeDuration dTimeDuration,
            float[] throughputPercents, 
            float[] sizePercents, 
            float[] timePercents,
            String businessProcess,
            String businessTransaction,
            String userGroup) throws CEMWebServicesException,RemoteException,Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
        
        DBusinessProcessDefinition bpDef = findBusinessProcessDefinition(cdo, businessProcess);
        long appId = bpDef.getApplicationId();
        String appName = bpDef.getApplicationName();
        long bpid = bpDef.getId();
        
        boolean isfound = false;
        long btid = 0L;
        String btName;
        DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
        for (int i = 0; i < busTranDef.length; i++){
            btid = busTranDef[i].getId();
            btName = busTranDef[i].getName();
            
            if (businessTransaction.compareTo(btName) == 0){
                isfound = true;
                break;
            }
        }
        
        if (isfound) {
            CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(sdo, throughputPercents, sizePercents, timePercents);
            String[][] keyPairs = new String[4][2];
            keyPairs[0][0] = String.valueOf(appId);
            keyPairs[0][1] = appName;
            keyPairs[1][0] = String.valueOf(bpid);
            keyPairs[1][1] = businessProcess;
            keyPairs[2][0] = String.valueOf(ugid);
            keyPairs[2][1] = userGroup;
            keyPairs[3][0] = String.valueOf(btid);
            keyPairs[3][1] = businessTransaction;
            
            List<DBusinessTransactionStats> results = 
                wsClientwrap.getBusinessTransactionStatsForUserGroup(btid, dTimeDuration, ugid);
            
            if (results.size() > 0)
                Util.writeBusinessTransactionStatisticToDataFile(keyPairs, results);    
            
        }else
            throw new Exception("business transaction name : " + businessTransaction + " is not found.");                
    }


    /**
     * Gets defects for specified business process
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param businessProcess
     *             The name of business process
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getDefectsDataByBusinessProcess(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration,
            String businessProcess) throws CEMWebServicesException,RemoteException,Exception
        {
            long bpid = findBusinessProcessDefinition(cdo, businessProcess).getId();
            
            CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
            List<DDefect> results = wsClientwrap.getDefectsByBusinessProcess(dTimeDuration, bpid);
            
            if (results.size() > 0)
                Util.writeDefectsToDataFile(results);
        }
    
    
    /**
     * Gets defects for specified business transaction
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param businessTransactionName
     *             The name of business transaction
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getDefectsDataByBusinessTransaction(
            ConfigurationDataOutServiceSoapBindingStub cdo,
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration,
            String businessTransactionName) throws CEMWebServicesException, RemoteException, Exception
        {
            long btid = 0l;
            boolean ismore = false;
            
            // get all BusinessProcessDefinitions
            DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
            CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
            for (int i = 0; i < allBusinessProcessDef.length; i++){
                long bpid = allBusinessProcessDef[i].getId();
                DBusinessTransactionDefinition[] busTranDef = cdo.getBusinessTransactionDefinitions(bpid);
                boolean isfound = false;
                
                for (int j = 0; j < busTranDef.length; j++){
                    btid = busTranDef[j].getId();
                    if (busTranDef[j].getName().compareTo(businessTransactionName) == 0){
                        isfound = true;
                        ismore = true;
                        break;
                    }
                }
                
                if (isfound) {
                    List<DDefect> results = wsClientwrap.getDefectsByBusinessTransaction(dTimeDuration, btid);
                    
                    if (results.size() > 0)
                        Util.writeDefectsToDataFile(results);
                }
            }
            
            if (!ismore)
                throw new Exception("business transaction name: " + businessTransactionName + " is not found.");
        }

    
    /**
     * Gets defects for specified user group
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval
     * @param userGroup
     *             The name of user group
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getDefectsdataByUserGroup(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration,
            String userGroup) throws CEMWebServicesException, RemoteException, Exception
    {
        long ugid = findUserGroupId(cdo, userGroup);
            
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
        List<DDefect> results = wsClientwrap.getDefectsByUserGroup(dTimeDuration, ugid);
        
        if (results.size() > 0)
            Util.writeDefectsToDataFile(results);
    }

    
    /**
     * Gets defects by defect type
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval
     * @param defectType
     *             The defect type
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getDefectsDataByDefectType(
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration,
            String defectType) throws CEMWebServicesException, RemoteException, Exception
    {
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
        List<DDefect> results = wsClientwrap.getDefectsByType(dTimeDuration, defectType);
        
        if (results.size() > 0)
            Util.writeDefectsToDataFile(results);
    }

    
    
    /**
     * Gets defects for specified business process, business transaction, user group and defect type
     * @param cdo
     *             ConfigurationDataOutServiceSoapBindingStub object 
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval 
     * @param businessProcess
     *             The name of business process
     * @param businessTransaction
     *             The name of business transaction
     * @param userGroup
     *             The name of user group
     * @param defectType
     *             The defect type
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void    getDefectsDataByBusinessProcess_BusinessTransaction_UserGroup_Type(
            ConfigurationDataOutServiceSoapBindingStub cdo, 
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration,
            String businessProcess,
            String businessTransaction,
            String userGroup,
            String defectType) throws CEMWebServicesException, RemoteException, Exception
    {
        long bpid = findBusinessProcessDefinition(cdo, businessProcess).getId();
        
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
        List<DDefect> results = wsClientwrap.getDefectsByBusinessProcess(dTimeDuration, bpid);
        
        if (results.size() > 0)
            Util.writeDefectsToDataFileByFilter(results, businessTransaction, userGroup, defectType);
    }

    
    /**
     * Gets incidents in time range from CEM web service
     * @param edo
     *             EventsDataOutServiceSoapBindingStub object
     * @param dTimeDuration
     *             The time range and the aggregation interval
     * @throws CEMWebServicesException
     * @throws RemoteException
     * @throws Exception
     * @since 1.0
     */
    public static void getIncidentsDataByTimeRange(
            EventsDataOutServiceSoapBindingStub edo,
            DTimeDuration dTimeDuration) throws CEMWebServicesException, RemoteException, Exception
    {
        CEMWSClientWrap wsClientwrap = new CEMWSClientWrap(edo);
        List<DIncident> results = wsClientwrap.getIncidentsByTime(dTimeDuration);
        
        if (results.size() > 0)
            Util.writeIncidentsToDataFile(results);
    }
    
    private static long findApplicationId(ConfigurationDataOutServiceSoapBindingStub cdo, String applicationName) throws Exception{
        boolean isfound = false;
        long appid = 0L;
        
        DApplicationDefinition[] allApplicationDef = cdo.getAllApplicationDefinitions();        
        for (int i = 0; i < allApplicationDef.length; i++) {            
            String bpName = allApplicationDef[i].getName();
            if (bpName.compareTo(applicationName) == 0){
                isfound = true;
                appid = allApplicationDef[i].getId();
                break;
            }
        }
        
        if (isfound == false){
            throw new Exception("business process name: " + applicationName + " is not found.");
        } else {
            return appid;
        }
    }
    
    private static long findUserGroupId(ConfigurationDataOutServiceSoapBindingStub cdo, String userGroupName) throws Exception {
        long ugid = 0L;
        boolean isFound = false;
        
        DUserGroup[] allUserGroupDef = cdo.getAllUserGroups();
        for (int i = 0; i < allUserGroupDef.length; i++){
            if (allUserGroupDef[i].getName().compareTo(userGroupName) == 0){
                ugid = allUserGroupDef[i].getId();
                isFound = true;
                break;
            }
        }
        
        if (isFound == false) {
            throw new Exception("user group name: " + userGroupName + " is not found.");
        } else {
            return ugid;
        }
    }
    
    private static long findUserId(ConfigurationDataOutServiceSoapBindingStub cdo, String userName) throws Exception {
           
            DUser user = cdo.getUserByName(userName);
            if(user == null) {
                throw new Exception("user : " + userName + " is not found.");
            } else {
                return user.getId();
            }
        }
    private static DBusinessProcessDefinition findBusinessProcessDefinition(ConfigurationDataOutServiceSoapBindingStub cdo, String businessProcessName) throws Exception {
        boolean isfound = false;
        DBusinessProcessDefinition foundBP = null;
        
        // get all BusinessProcessDefinitions
        DBusinessProcessDefinition[] allBusinessProcessDef = cdo.getAllBusinessProcessDefinitions();
        for (int i = 0; i < allBusinessProcessDef.length; i++) {            
            if (allBusinessProcessDef[i].getName().compareTo(businessProcessName) == 0){
                isfound = true;
                foundBP = allBusinessProcessDef[i];
                break;
            }
        }
        
        if (isfound == false){
            throw new Exception("business process name: " + businessProcessName + " is not found.");
        } else {
            return foundBP;
        }
    }
}
